/*
 *	Copyright (C) 2007, 2008, 2009. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <iostream>
#include <qvmath.h>
#include <qvmath/qvrationalnumber.h>

double norm2(const QPointF &p)
	{
	return sqrt(p.x() * p.x() + p.y() * p.y());
	}

const int qvFactorial(const int n)
	{
	int result = 1;
	for (int i = 2; i <= n; i++)
		result *= i;

	return result;
	};

const double qvFactorialDivisionDouble(const int numerator, const int denominator)
	{
	double result = 1;
	for (int i = denominator+1; i <= numerator; i++)
		result *= i;

	return result;
	}

const double qvCombination(const int setRange, const int subsetRange)
	{
	Q_ASSERT(setRange > subsetRange);

	QVRationalNumber subSetFactorial, divisionFactorial;

	for (int i = 2; i <= subsetRange; i++)
		subSetFactorial.mult(i);

	for (int i = setRange - subsetRange + 1; i <= setRange; i++)
		divisionFactorial.mult(i);
	
	return divisionFactorial/subSetFactorial;
	}

const double qvAngle(const QPointF &point)
	{
	const double x = point.x(), y = point.y();

	if (x>0)
		if (y>=0)
			return atan(y/x);
		else
			return atan(y/x) + 2*PI;
	else if (x == 0)
		if (y>0)
			return PI/2;
		else
			return 3*PI/2;
	else // x < 0
		return atan(y/x)+PI;
	}

const int qvRandom(const int minValue, const int maxValue)
	{ return rand()%(maxValue-minValue+1) + minValue; }

// This function returns the positive degrees for a given angle
const double qvClockWiseAngle(const QPointF &point1, const QPointF &point2)
	{
	//const double clockAngle = qvAngle(point2) - qvAngle(point1);
	//return (clockAngle < 0)? clockAngle + 2*PI:clockAngle;
	double dtheta = atan2(point2.y(), point2.x()) - atan2(point1.y(), point1.x());
	
	while (dtheta > PI)
		dtheta -= 2*PI;
	while (dtheta < -PI)
		dtheta += 2*PI;
	
	return dtheta;
	}

